
/*
 * de.unkrig.loggifier - Injects logging code into JAVA[TM] class files
 *
 * Copyright (c) 2011, Arno Unkrig
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided that the
 * following conditions are met:
 *
 *    1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *       following disclaimer.
 *    2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the
 *       following disclaimer in the documentation and/or other materials provided with the distribution.
 *    3. The name of the author may not be used to endorse or promote products derived from this software without
 *       specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package de.unkrig.loggifier;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Set;

import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.annotation.processing.SupportedSourceVersion;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.Element;
import javax.lang.model.element.TypeElement;
import javax.tools.FileObject;
import javax.tools.StandardLocation;

import de.unkrig.commons.nullanalysis.NotNullByDefault;

/**
 * @see <a href="http://docs.oracle.com/javase/1.5.0/docs/guide/apt/GettingStarted.html">Getting Started with APT</a>
 */
@SupportedAnnotationTypes("*") @SupportedSourceVersion(SourceVersion.RELEASE_6) @NotNullByDefault(false) public
class AnnotationProcessor extends AbstractProcessor {

    @Override public boolean
    process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnvironment) {

        System.out.println(roundEnvironment.toString());

        System.out.println("Root elements:");
        this.printElements(roundEnvironment.getRootElements(), "  ");

        for (TypeElement annotation : annotations) {
            System.out.println("Annotation '" + annotation + "':");
            this.printElements(roundEnvironment.getElementsAnnotatedWith(annotation), "  ");
        }

        for (Element rootElement : roundEnvironment.getRootElements()) {
            System.out.println("Instrumenting '" + rootElement + "':");
            if (rootElement instanceof TypeElement) {
                TypeElement typeElement = (TypeElement) rootElement;
                try {
                    String       qn  = typeElement.getQualifiedName().toString();
                    int          idx = qn.lastIndexOf('.');
                    CharSequence pkg = idx == -1 ? null : qn.subSequence(0, idx);
                    FileObject   cl  = this.processingEnv.getFiler().getResource(
                        StandardLocation.CLASS_OUTPUT,
                        pkg,
                        rootElement.getSimpleName()
                    );
                    System.out.println(cl.getName());
                    InputStream is = cl.openInputStream();
                    try {
                    } finally {
                        try { is.close(); } catch (Exception e) {}
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
//        roundEnvironment.getRootElements().iterator().next().
        return true;
    }

    private void
    printElements(Collection<? extends Element> elements, String prefix) {
        for (Element element : elements) {
            System.out.println(prefix + element + " (" + element.getClass() + ")");
            this.printElements(element.getEnclosedElements(), prefix + "  ");
        }
    }
}
